// SPDX-License-Identifier: MPL-2.0
// (c) 2021 Pascal Monasse <pascal.monasse@enpc.fr>
// Compute gradient L1 norm of image

#include "io_png.h"
#include <iostream>
#include <cmath>
#include <cstring>

inline unsigned char clip(float v) {
    return (v>255.0f)? 255: (unsigned char)v;
}

int main(int argc, char** argv) {
    if(argc!=3) {
        std::cerr << "Usage: " << argv[0]
                  << " imageIn.png gradientOut.png" << std::endl;
        return 1;
    }
    size_t nx, ny;
    float* imgIn = io_png_read_f32_gray(argv[1], &nx, &ny);
    if(!imgIn) {
        std::cerr << "Error reading image file " << argv[1] << std::endl;
        return 1;
    }

    const size_t n = nx*ny;
    unsigned char* imgOut = new unsigned char[n];

    // smooth
    float* in=imgIn+n-1;
    for(size_t y=ny-1; y>0; y--, in--)
        for(size_t x=nx-1; x>0; x--, in--)
            *in = (in[-1]+in[0]+in[-ny]+in[-ny-1])/4;

    // gradient
    in=imgIn;
    unsigned char* out=imgOut;
    if(nx>1 && ny>1)
        for(size_t y=0; y<ny; y++)
            for(size_t x=0; x<nx; x++, in++) {
                float dx = (x+1<nx)? in[1]-in[0]: in[0]-in[-1];
                float dy = (y+1<ny)? in[ny]-in[0]: in[0]-in[-ny];
                *out++ = clip(fabsf(dx) + fabsf(dy));
            }
    else
        memset(out, 0, n);

    if(io_png_write_u8(argv[2], imgOut, nx, ny, 1) !=0) {
        std::cerr << "Error writing image file " << argv[2] << std::endl;
        return 1;
    }

    free(imgIn);
    delete [] imgOut;
    return 0;
}
